//
//	MPI_API.cpp -       .
//

#include "MPI_Memory.h"
#include "MPI_DL11.h"
#include "MPI_VM1_V.h"
#include "MPI_1801VM1.h"
#include "MPI_VE_Timer.h"
#include "MPI_HD.h"
#include "MPI_BIN.h"
#include "MPI_LTC.h"

// #define USE_V_MODEL

#define EM_LOG(x) { FILE *MPI_log=fopen( "MPI.log", "a"); fprintf(MPI_log, x ); fclose( MPI_log ); }


MPI_AdapterHooks  Hooks;
VM1_bus_type      VM1_bus;

int nCLK = 0;

MPI_RAM     HALT_RAM( 0177600, 64 );
MPI_ROM		HALT_ROM( 0160000, 014000, "000.dat" );
MPI_RAM     RAM( 000000, 0160000 );
//MPI_SEL1	SEL1( 0160001 );							//     160000
MPI_HD      HD( "HDSYS.DSK", RAM(), true );
MPI_SEL1	SEL1( 0000000 );							//     000000
//MPI_BIN		BIN("791401.bin", RAM() );
//MPI_BIN		BIN("IRQtst.bin", RAM() );
MPI_DL11	DL11;
MPI_LTC		LTC;
MPI_VM1_V	VM1_V;
MPI_1801VM1 VM1_A;


void PrintBusState()
{
	if( !(nCLK%50) && !VM1_bus.CLC )
	{
		EM_LOG( "------------------------------------------------------------\n" );
		EM_LOG( "          : C        B S D D W R B I I D D S I H E A D I S S\n" );
		EM_LOG( "   nCLK   : L   AD   S Y I O T P S R A M M A N A V C C R L L\n" );
		EM_LOG( "          : C        Y N N U B L 7 Q K R G C I L N L L 3 1 2\n" );
		EM_LOG( "------------------------------------------------------------\n" );
	}

	char str[512];
	char *pChar = str + 12;
	
	if( !VM1_bus.CLC )
	{
		sprintf(str,"%10u: 0 %06o 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0\n", nCLK, word(~VM1_bus.AD) );
	}
	else
	{
		sprintf(str,"          : 0 %06o 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0\n", word(~VM1_bus.AD) );
	}
	
	if( !VM1_bus.CLC ) { *pChar = '1'; }

	pChar += 9;
	
	if( !VM1_bus.BSY ) { *pChar = '1'; }
	
	pChar += 2;
	
	if( !VM1_bus.SYNC ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.DIN ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.DOUT ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.WTBT ) { *pChar = '1'; }
	
	pChar += 2;
	
	if( !VM1_bus.RPLY ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.BS7 ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.IRQ ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.IAK ) { *pChar = '1'; }
	
	pChar += 2;
	
	if( !VM1_bus.DMR ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.DMG ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.SACK ) { *pChar = '1'; }
	
	pChar += 2;
	
	if( !VM1_bus.INIT ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.HALT ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.EVNT ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.ACLO ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.DCLO ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.IRQ3 ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.SEL1 ) { *pChar = '1'; }

	pChar += 2;
	
	if( !VM1_bus.SEL2 ) { *pChar = '1'; }
	
	EM_LOG(str);
}


void Clock()
{
	VM1_bus.CLC = false;

#ifdef USE_V_MODEL
	((MPI_Interface*)&VM1_V   )->Clock();
#else
	((MPI_Interface*)&VM1_A   )->Clock();
#endif
	((MPI_Interface*)&LTC     )->Clock();
	((MPI_Interface*)&DL11    )->Clock();
	((MPI_Interface*)&SEL1    )->Clock();
	((MPI_Interface*)&HALT_RAM)->Clock();
	((MPI_Interface*)&HALT_ROM)->Clock();
	((MPI_Interface*)&RAM     )->Clock();
	((MPI_Interface*)&HD      )->Clock();

//	PrintBusState();

	VM1_bus.CLC = true;

#ifdef USE_V_MODEL
	((MPI_Interface*)&VM1_V   )->Clock();
#else
	((MPI_Interface*)&VM1_A   )->Clock();
#endif
	((MPI_Interface*)&LTC     )->Clock();
	((MPI_Interface*)&DL11    )->Clock();
	((MPI_Interface*)&SEL1    )->Clock();
	((MPI_Interface*)&HALT_RAM)->Clock();
	((MPI_Interface*)&HALT_ROM)->Clock();
	((MPI_Interface*)&RAM     )->Clock();
	((MPI_Interface*)&HD      )->Clock();

//	PrintBusState();

	VM1_bus.qwClocksCount++;

	nCLK++;
}

void Clocks( dword uClocks = 1 )
{
	for( dword i = 0 ; i < uClocks ; i++ )
	{
		Clock();
	}
}

int main( int argc, char **argv )
{
	printf("MPI_API   v1.1  2017\n");

	FILE *MPI_log=fopen( "MPI.log", "w" ); fclose( MPI_log );
	
	((MPI_Interface*)&SEL1    )->Link_MPI( &VM1_bus, &Hooks );
	((MPI_Interface*)&HALT_RAM)->Link_MPI( &VM1_bus, &Hooks );
	((MPI_Interface*)&HALT_ROM)->Link_MPI( &VM1_bus, &Hooks );
	((MPI_Interface*)&RAM     )->Link_MPI( &VM1_bus, &Hooks );
	((MPI_Interface*)&DL11    )->Link_MPI( &VM1_bus, &Hooks );
	((MPI_Interface*)&LTC     )->Link_MPI( &VM1_bus, &Hooks );
	((MPI_Interface*)&HD      )->Link_MPI( &VM1_bus, &Hooks );
	
#ifdef USE_V_MODEL
	((MPI_Interface*)&VM1_V   )->Link_MPI( &VM1_bus, &Hooks );
#else
	((MPI_Interface*)&VM1_A   )->Link_MPI( &VM1_bus, &Hooks );
#endif
	

	/////////////////////////////////////////////////////////////

	nCLK = 0;

	VM1_bus.DCLO = false;
	VM1_bus.ACLO = false;
	
	Clocks( 4 );
	
	VM1_bus.DCLO = true;	

	Clocks( 8 );
	
	VM1_bus.ACLO = true;
	
	/////////////////////////////////////////////////////////////

//	while( nCLK < 50000 )
	while( true )
	{
		Clock();
//if( nCLK > 9500000 ) { printf("%i\r", nCLK ); }
	}
	
	return 0;
}