//
//  MPI_HD.cpp
//

#include "MPI_HD.h"

#include <conio.h>

// #define L(x) pHooks->DisAsmStrOut(x,false)
#define L(x) { FILE *MPI_log=fopen( "MPI.log", "a"); fprintf(MPI_log, x ); fclose( MPI_log ); }

//
//

#define  HD_CheckUnit     0
#define  HD_SetUnit       1
#define  HD_SetBlockNum   2
#define  HD_SetBufAddr    3
#define  HD_SetWordCount  4
#define  HD_Read          5
#define  HD_Write         6
#define  HD_Get_Unit_Size 7

//

MPI_HD::MPI_HD( const char *acDSK_File, const byte *pMemory, bool bReadBoot_HD0 ):
MPI_DeviceClass( 4 ) //  RPLY = 4 .
{
	abReadAccess[  HDCMD ] = true;
	abWriteAccess[ HDCMD ] = true;
	
	abReadAccess[  HDCMD+1 ] = true;
	abWriteAccess[ HDCMD+1 ] = true;
	
	abReadAccess[  HDDAT ] = true;
	abWriteAccess[ HDDAT ] = true;
	
	abReadAccess[  HDDAT+1 ] = true;
	abWriteAccess[ HDDAT+1 ] = true;
	
	wHDCMD = 0;
	wHDDAT = 0;
	
	aMemory = (byte*)pMemory;
	
	if( MountHdUnit( 0, acDSK_File )  &&  bReadBoot_HD0 )
	{
		HDReadDSK( 0, 0, 256, 0 );  //   0  0   0
	}
}

void MPI_HD::IO_fun( IO_ModeType IO_Mode, IO_SizeType IO_Size, int nIO_Addr, word & wData )
{
	if( IO_Mode == IO_READ )
	{
		switch( nIO_Addr )
		{
			case HDCMD:
			case HDCMD|1:
			{
				if( IO_Size == IO_WORD ) 
				{
					wData = wHDCMD;
				}
				else
				{
					byte*   pByte = (byte*) & wHDCMD;
					wData = pByte[ nIO_Addr &1 ];
				}
			}
			break;

			case HDDAT:
			case HDDAT|1:
			{
				if( IO_Size == IO_WORD ) 
				{
					wData = wHDDAT;
				}
				else
				{
					byte*   pByte = (byte*) & wHDDAT;
					wData = pByte[ nIO_Addr &1 ];
				}
			}
			break;
		}
	}
	else
	if( IO_Mode == IO_WRITE )
	{
		switch( nIO_Addr )
		{
			case HDCMD:
			case HDCMD|1:
			{
				if( IO_Size == IO_WORD ) 
				{
					wHDCMD = wData;
				}
				else
				{
					byte* pByte = (byte*) & wHDCMD;
					pByte[ nIO_Addr &1 ] = (byte)wData;
				}

				HD_device_command( wHDCMD & 0xFF );
			}
			break;

			case HDDAT:
			case HDDAT|1:
			{
				if( IO_Size == IO_WORD ) 
				{
					wHDDAT = wData;
				}
				else
				{
					byte* pByte = (byte*) & wHDDAT;
					pByte[ nIO_Addr &1 ] = (byte)wData;
				}
			}
			break;
		}
	}
}

int MPI_HD::HDReadDSK( word nHDnum, word nBlockNum, word nWordCount, word nEmulBufAddr )
{
  long nByteNum = 512;
  word nBytes = nWordCount*2;

  nByteNum *= nBlockNum;
/*
{
char str[512];
sprintf( str, "HDReadDSK : = Unit: %u ; Block: %u ; WordCount: %u ; BufAddr: %06o\n", nHDnum, nBlockNum, nWordCount, nEmulBufAddr );
L( str );
}//*/
  if( hDSK[nHDnum] )
  {
    if( nBlockNum + nWordCount/256 > anDSKlen[nHDnum] )
    {
      nBytes = (anDSKlen[nHDnum] - nBlockNum )*512;
    }

    lseek( hDSK[nHDnum], nByteNum, SEEK_SET );
    read(  hDSK[nHDnum], aMemory + nEmulBufAddr, nBytes );
    return 1;
  }

  return 0;
}

int MPI_HD::HDWriteDSK( word nHDnum, word nBlockNum, word nWordCount, word nEmulBufAddr )
{
  unsigned long nByteNum = 512;
  word  nBytes = nWordCount*2;

  nByteNum *= nBlockNum;
/*
{
char str[512];
sprintf( str, "HDWriteDSK : = Unit-[%u] ; Block-[%u] ; WordCount-[%u] ; BufAddr: %06o\n", nHDnum, nBlockNum, nWordCount, nEmulBufAddr );
L( str );
}//*/
  if( hDSK[nHDnum] )
  {
    if( nBlockNum + nWordCount/256 > anDSKlen[nHDnum] )
    {
      nBytes = (anDSKlen[nHDnum] - nBlockNum )*512;
    }

    lseek( hDSK[nHDnum], nByteNum, SEEK_SET );
    write( hDSK[nHDnum], aMemory + nEmulBufAddr, nBytes );
    return 1;
  }

  return 0;
}

int MPI_HD::MountHdUnit( word nHDnum, const char* acDSK_File )
{
	long  lFileLen;

    hDSK[nHDnum] = open( acDSK_File, O_RDWR	| O_BINARY );

    if( hDSK[nHDnum] <= 0 )
    {
      printf("HD: DSK file not found--[%s]\n", acDSK_File );
      return 0;
    }
    else
    {
      lFileLen = filelength( hDSK[nHDnum] );
      anDSKlen[nHDnum] = lFileLen/512;
	}
   
	printf("HD%i attached to [%s] \n", nHDnum, acDSK_File );
	return 1;
}

void MPI_HD::HD_device_command( word cmd )
{
   switch( cmd )
   { ////
     case HD_SetUnit:
	 {
	   if( wHDDAT > 7 )
	   {
         wHDCMD |= 0100000;   //  .
         break;
	   }
	 
       HD_Unit = wHDDAT;

       if( hDSK[HD_Unit] <= 0 )
       {
         wHDCMD |= 0100000;   //  .
       }
       else
       {
         wHDCMD &= ~0100000;   //  .
       }
	 }
/*
{
char str[512];
sprintf( str, "HD_SetUnit:: HD_Unit: %u \n", nHDnum );
L( str );
}//*/
     break;


     ////
     case HD_SetBlockNum:
	 {
       if( HD_BlockNum > anDSKlen[HD_Unit] - 1 )
       {
         wHDCMD |= 0100000;   //  .
         break;
       }

       HD_BlockNum = wHDDAT;
/*
{
char str[512];
sprintf( str, "HD_SetBlockNum:: HD_BlockNum: %u \n", HD_BlockNum );
L( str );
}//*/
	 }
     break;


     ////
     case HD_SetBufAddr:
	 {
       HD_BufAddr = wHDDAT;
/*
{
char str[512];
sprintf( str, "HD_SetBufAddr:: HD_BlockNum: %u \n", HD_BufAddr );
L( str );
}//*/
	 }
     break;


     ////
     case HD_SetWordCount:
	 {
       HD_WordCount = wHDDAT;

       int x = HD_BlockNum + HD_WordCount/256 - 1;
       if( x < 0 ) { x = 0; }

       if( x > anDSKlen[HD_Unit] - 1 )
       {
         wHDCMD |= 0100000;   //  .
       }
/*
{
char str[512];
sprintf( str, "HD_SetWordCount:: HD_WordCount: %u \n", HD_WordCount );
L( str );
}//*/
	 }
     break;


     ////
     case HD_Read:
	 {
       int  ok = HDReadDSK( HD_Unit, HD_BlockNum, HD_WordCount, HD_BufAddr );

       if( !ok )
       {
         wHDCMD |= 0100000;   //  .
       }
	 }
     break;


     ////
     case HD_Write:
	 {
       int  ok = HDWriteDSK( HD_Unit, HD_BlockNum, HD_WordCount, HD_BufAddr );

       if( !ok )
       {
         wHDCMD |= 0100000;   //  .
       }
	 }
     break;


     ////
     case HD_Get_Unit_Size:
	 {
       if( (HD_Unit >= 0) && (HD_Unit <= 7) )
       {
         wHDDAT = anDSKlen[HD_Unit];
       }
       else
       {
         wHDDAT = 0;
       }
	 }
     break;
   }
}



