//
//  MPI_Memory.cpp
//

#include "MPI_Memory.h"


// #define L(x) pHooks->DisAsmStrOut(x,false)
#define L(x) { FILE *MPI_log=fopen( "MPI.log", "a"); fprintf(MPI_log, x ); fclose( MPI_log ); }

//
//

//

MPI_RAM::MPI_RAM( word wStartAddr, word wRamSizeBytes ):
MPI_DeviceClass( 6 ) //  RPLY = 6 .
{
	memset( aRAM, 0, sizeof(aRAM) );

	for( word addr = wStartAddr ; addr < wStartAddr + wRamSizeBytes ; addr++ )
	{
		abReadAccess[  addr ] = true;
		abWriteAccess[ addr ] = true;
	}
}

void MPI_RAM::IO_fun( IO_ModeType IO_Mode, IO_SizeType IO_Size, int nIO_Addr, word & wData )
{
	if( IO_Mode == IO_READ )
	{
		if( IO_Size == IO_WORD ) { wData = *(word*)(&aRAM[ nIO_Addr &(~1) ]); }
		else                     
		if( IO_Size == IO_BYTE ) { wData = aRAM[ nIO_Addr ]; }
	}
	else
	if( IO_Mode == IO_WRITE )
	{
		if( IO_Size == IO_WORD ) { *(word*)(&aRAM[ nIO_Addr &(~1) ]) = wData; }
		else
		if( IO_Size == IO_BYTE ) { aRAM[ nIO_Addr ] = (byte)wData; }
	}
}

////////////////////////////////////////////////////////////////////////////////////

MPI_ROM::MPI_ROM( word wStartAddr, word wRomSizeBytes, const char *acBIN_File ):
MPI_DeviceClass( 2 ) //  RPLY = 2 .
{
	memset( aROM, 0, sizeof(aROM) );

	int hFile = open( acBIN_File, O_RDONLY | O_BINARY );

	if( hFile < 0 )
	{
		printf("ERROR: User ROM data file NOT found: [%s] \n", acBIN_File );
	}
	else
	{
		read( hFile, &aROM[wStartAddr], wRomSizeBytes );
		close(hFile);
	}
	
	for( word addr = wStartAddr ; addr < wStartAddr + wRomSizeBytes ; addr++ )
	{
		abReadAccess[ addr ] = true;
	}
}

void MPI_ROM::IO_fun( IO_ModeType IO_Mode, IO_SizeType IO_Size, int nIO_Addr, word & wData )
{
	if( IO_Mode == IO_READ )
	{
		if( IO_Size == IO_WORD ) { wData = *(word*)(&aROM[ nIO_Addr &(~1) ]);    }
		else                     
		if( IO_Size == IO_BYTE ) { wData = aROM[ nIO_Addr ]; }
	}
}

////////////////////////////////////////////////////////////////////////////////////

MPI_SEL1::MPI_SEL1( word wStartAddr ):
MPI_DeviceClass( 0, 0 ) //  RPLY = 0  ON, 0  OFF.
{
	abReadAccess[  0177716 ] = true;
	abWriteAccess[ 0177716 ] = true;

	abReadAccess[  0177717 ] = true;
	abWriteAccess[ 0177717 ] = true;
	
	wSEL1 = wStartAddr|BIT_2;
}

 
void MPI_SEL1::IO_fun( IO_ModeType IO_Mode, IO_SizeType IO_Size, int nIO_Addr, word & wData )
{
	if( IO_Mode == IO_READ )
	{
		if( IO_Size == IO_WORD ) { wData = wSEL1; }
		else                     
		if( IO_Size == IO_BYTE ) { wData = ((byte*)&wSEL1)[ nIO_Addr &1 ]; }
	}
	else
	if( IO_Mode == IO_WRITE )
	{
		word wSEL1_Prev = wSEL1;

		wSEL1_Prev &= ~( BIT_3|BIT_2 );
	
		if( IO_Size == IO_WORD ) { *((byte*)&wSEL1) = (byte) wData; }
		else
		if( IO_Size == IO_BYTE ) 
		{
			if( nIO_Addr == 0177716 )
			{
				*((byte*)&wSEL1) = (byte) wData;
			}
		}
		
		wSEL1 &= BIT_3|BIT_2;
		wSEL1 |= wSEL1_Prev;
	}
}
///