//
//  MPI_model.cpp
//

#include "MPI_model.h"

#pragma warning(disable:4996)

// extern HINSTANCE hInstDLL;
// extern char acModulePath[];

// SetComponentBuildInternal( MPI_model );

//////////////////////////////////////////

// #define L(x) pHooks->DisAsmStrOut(x,false)
#define L(x) { FILE *MPI_log=fopen( "MPI.log", "a"); fprintf(MPI_log, x ); fclose( MPI_log ); }


//////////////////////////////////////////

void MPI_DeviceClass::Clock()
{
	if( !pVM1_bus->CLC )
	{
		return;
	}
	
	///////////////////////////

	bool DCLO = pVM1_bus->DCLO;

	if( DCLO != DCLO_Prev )
	{
		DCLO_Prev = DCLO;
		
		if( DCLO ) { Power_ON();  }
		else       { Power_OFF(); }
	}

	///////////////////////////

	bool INIT = pVM1_bus->INIT;
	
	if( INIT != INIT_Prev )
	{
		INIT_Prev = INIT;
		
		if( !INIT )
		{
			Init();
		}
	}
	
	///////////////////////////

	if( wVector  &&  !bIRQ_ControllerActive  &&  pVM1_bus->IAK )
	{
		if( IRQ_User_ID > pVM1_bus->IRQ_ID )
		{
			pVM1_bus->IRQ = 0;
			pVM1_bus->IRQ_ID = IRQ_User_ID;
		}
	}

	///////////////////////////

	if( !BusCycleController )
	{
		if( wVector && !pVM1_bus->IAK )
		{
			bIRQ_ControllerActive = true;
			BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::IRQ_BusCycleController_1 );
			nBusCycleDelay = 1;
		}
		else
		if( !pVM1_bus->SYNC )
		{
			BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::SYNC_ON_BusCycleController );
		}
		else
		{
			wAddr = ~ pVM1_bus->AD;
			WTBT_Prev = pVM1_bus->WTBT;
		}
	}

	///////////////////////////

	if( BusCycleController )
	{
		BusCycleController();
	}
}

void MPI_DeviceClass::IRQ_BusCycleController_1()
{
	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}
	
	pVM1_bus->IRQ = ~0;
	
	nBusCycleDelay = 1; //     IRQ   RPLY.
	
	BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::IRQ_BusCycleController_2 );
}
void MPI_DeviceClass::IRQ_BusCycleController_2()
{
	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}
	
	if( wVector )
	{
		pVM1_bus->RPLY = false;
		pVM1_bus->AD   = ~wVector;
		
		wVector = 0;
		
		nBusCycleDelay = nRPLY_OFF_Latency_CLK;

		BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::IRQ_BusCycleController_3 );
	}
	else
	{
		BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::IRQ_BusCycleController_4 );
	}
}
void MPI_DeviceClass::IRQ_BusCycleController_3()
{
	if( !pVM1_bus->DIN )
	{
		return;
	}

	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}

	pVM1_bus->RPLY = true;
	pVM1_bus->AD   = ~0;
	
	BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::IRQ_BusCycleController_4 );
}
void MPI_DeviceClass::IRQ_BusCycleController_4()
{
	if( !pVM1_bus->IAK )
	{
		return;
	}

	bIRQ_ControllerActive = false;
	BusCycleController = NULL;

	pVM1_bus->IRQ_ID = 0;
}
  
void MPI_DeviceClass::SYNC_ON_BusCycleController()
{
//	wAddr = ~ pVM1_bus->AD;	//  ReplyLatency == 0 -    -      .
	
	nBusCycleDelay = nRPLY_ON_Latency_CLK;

	if( WTBT_Prev )
	{
		if( !abReadAccess[ wAddr ] )
		{
			BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::SYNC_OFF_BusCycleController );
			return;
		}
		BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::DIN_ON_BusCycleController );
		DIN_ON_BusCycleController();
	}
	else
	{
		if( !abWriteAccess[ wAddr ] )
		{
			BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::SYNC_OFF_BusCycleController );
			return;
		}
		BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::DOUT_ON_BusCycleController );
		DOUT_ON_BusCycleController();
	}
}
void MPI_DeviceClass::SYNC_OFF_BusCycleController()
{
	if( !pVM1_bus->SYNC )
	{
		return;
	}

	BusCycleController = NULL;

	if( !pVM1_bus->BSY )
	{
		wAddr = ~ pVM1_bus->AD;
		WTBT_Prev = pVM1_bus->WTBT;
	}
}
void MPI_DeviceClass::SYNC_OFF_BusCycleController_1()
{
	if( !pVM1_bus->SYNC )
	{
		if( !pVM1_bus->DOUT )
		{//  DATIO  DATIOB
			nBusCycleDelay = nRPLY_ON_Latency_CLK;
			BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::DOUT_ON_BusCycleController );
			DOUT_ON_BusCycleController();
		}
		return;
	}

	BusCycleController = NULL;

	if( !pVM1_bus->BSY )
	{
		wAddr = ~ pVM1_bus->AD;
		WTBT_Prev = pVM1_bus->WTBT;
	}
}
void MPI_DeviceClass::DIN_ON_BusCycleController()
{
	if( pVM1_bus->DIN )
	{
		return;
	}

	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}

	word wData;

	IO_fun( IO_READ, IO_WORD, wAddr, wData );
		
	pVM1_bus->AD   = ~ wData;
	pVM1_bus->RPLY = false;
	
	nBusCycleDelay = nRPLY_OFF_Latency_CLK;
	
	BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::DIN_OFF_BusCycleController );
}
void MPI_DeviceClass::DIN_OFF_BusCycleController()
{
	if( !pVM1_bus->DIN )
	{
		return;
	}

	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}
	
	pVM1_bus->AD   = ~0;
	pVM1_bus->RPLY = true;
	
	BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::SYNC_OFF_BusCycleController_1 );
}
void MPI_DeviceClass::DOUT_ON_BusCycleController()
{
	if( pVM1_bus->DOUT )
	{
		return;
	}

	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}

	word wData = ~ pVM1_bus->AD;

	if( pVM1_bus->WTBT )
	{
		IO_fun( IO_WRITE, IO_WORD, wAddr, wData );
	}
	else
	{
		if( wAddr & 1 ) { wData >>= 8;      }
		else            { wData  &= 0x00FF; }

		IO_fun( IO_WRITE, IO_BYTE, wAddr, wData );
	}
		
	pVM1_bus->RPLY = false;

	nBusCycleDelay = nRPLY_OFF_Latency_CLK;
	
	BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::DOUT_OFF_BusCycleController );
}
void MPI_DeviceClass::DOUT_OFF_BusCycleController()
{
	if( !pVM1_bus->DOUT )
	{
		return;
	}
	
	if( nBusCycleDelay > 0 )
	{
		nBusCycleDelay--;
		return;
	}
	
	pVM1_bus->RPLY = true;
	
	BusCycleController = fastdelegate::MakeDelegate(this, &MPI_DeviceClass::SYNC_OFF_BusCycleController );
}

//////////////////////////////////////////
