//
//  MPI_model.h  -      ( ..      )
//                      .
//

#ifndef MPI_modelDefs
#define MPI_modelDefs 1

#include "sysmac.h"
#include "FastDelegate.h"

#pragma warning(disable:4996)


///////////////////////

//    

struct  MPI_bus_type 
{
	char  const * const    Type;
	qword                  qwClocksCount;
	///////////////////////////
	bool                   CLC;
	///////////////////////////
	word                   AD;
	byte                   Aext;
	bool                   SYNC;
	bool                   RPLY;
	bool                   DIN;
	bool                   DOUT;
	bool                   WTBT;
	bool                   BS7;
	///////////////////////////
	bool                   REF;      // Memory refresh
	bool                   INIT;
	bool                   HALT;
	bool                   EVNT;
	bool                   ACLO;
	bool                   DCLO;
	///////////////////////////
	byte                   IRQ;
	word                   IRQ_ID;  //  ,  ID ,  IRQ.
	bool                   IAK;
	///////////////////////////
	bool                   DMR;
	bool                   DMG;
	bool                   SACK;
	///////////////////////////
	bool                 & DMGI;     //   DMG
	bool                 & DMGO;     //   DMG
	bool                 & IAKI;     //   IAK
	bool                 & IAKO;     //   IAK

	MPI_bus_type(): 
	Type("MPI"),
	qwClocksCount(0),
	CLC ( true ),
	AD  ( ~0 ),
	Aext( ~0 ),
	SYNC( true ),
	RPLY( true ),
	DIN ( true ),
	DOUT( true ),
	WTBT( true ),
	BS7 ( true ),
	REF ( true ),
	INIT( true ),
	HALT( true ),
	EVNT( true ),
	ACLO( true ),
	DCLO( true ),
	IRQ ( ~0 ),
	IRQ_ID(0),
	IAK ( true ),
	DMR ( true ),
	DMG ( true ),
	SACK( true ),
	DMGI( DMG ),
	DMGO( DMG ),
	IAKI( IAK ),
	IAKO( IAK )
	{}

protected:
	MPI_bus_type( const char *acType ): Type( acType ),
	CLC ( true ),
	AD  ( ~0 ),
	Aext( ~0 ),
	SYNC( true ),
	RPLY( true ),
	DIN ( true ),
	DOUT( true ),
	WTBT( true ),
	BS7 ( true ),
	REF ( true ),
	INIT( true ),
	HALT( true ),
	EVNT( true ),
	ACLO( true ),
	DCLO( true ),
	IRQ ( ~0 ),
	IAK ( true ),
	DMR ( true ),
	DMG ( true ),
	SACK( true ),
	DMGI( DMG ),
	DMGO( DMG ),
	IAKI( IAK ),
	IAKO( IAK )
	{}
};



///////////////////////

//    18011


struct  VM1_bus_type : public MPI_bus_type 
{
	bool                   SEL1;
	bool                   SEL2;
	bool                   BSY;
	bool                   IRQ3;
	bool                 & IRQ1;    //   HALT
	bool                 & IRQ2;    //   EVNT
	byte                 & VIRQ;    //   IRQ
	
	VM1_bus_type(): MPI_bus_type("VM1"),
	SEL1( true ),
	SEL2( true ),
	BSY ( true ),
	IRQ3( true ),
	IRQ1( HALT ),
	IRQ2( EVNT ),
	VIRQ( IRQ )
	{}
};


/////////////////////////////////////////////////////////////


struct MPI_AdapterHooks
{
//	fastdelegate::FastDelegate2< const char *, bool >  DisAsmStrOut;
	int	STUB;	//      .
};


/////////////////////////////////////////////////////////////

//    MPI_Interface -     

class MPI_Interface {
public:
	virtual void   Clock()=0;
	virtual void   Link_MPI( MPI_bus_type *pMPI, MPI_AdapterHooks *pHooks )=0;

protected:
	virtual ~MPI_Interface(){}
	MPI_Interface(){}
};

///////////////////////////////
class qword_ref {
public:
	virtual ~qword_ref(){}
	
	qword_ref():
	pArg(NULL)
	{}

	void Set_pArg( qword & ARG ) { pArg = &ARG; };

	virtual qword operator()(void)const
	{
		if( pArg ) { return *pArg; }
		return 0;
	}
	inline  operator qword() const{ return operator()(); }
	
protected:
	qword *pArg;
};
///////////////////////////////

//    MPI_DeviceClass -      INIT   DATI, DATO, DATOB, DATIO, DATIOB, Power_ON, Power_OFF
//                        .          MPI_Device.

class MPI_DeviceClass : public MPI_Interface {
public:
	virtual ~MPI_DeviceClass(){}
	
	MPI_DeviceClass( int nRPLY_ON_Latency_Clocks, int nRPLY_OFF_Latency_Clocks = 2 ):
	nRPLY_ON_Latency_CLK(  nRPLY_ON_Latency_Clocks ),
	nRPLY_OFF_Latency_CLK( nRPLY_OFF_Latency_Clocks ),
	IRQ_User_ID(0),
	bIRQ_ControllerActive(false),
	WTBT_Prev(true),
	DCLO_Prev(true),
	INIT_Prev(true)
	{
		memset(  abReadAccess, false, 0x10000 );
		memset( abWriteAccess, false, 0x10000 );
	}

protected:
	////////////////////////////
	
	VM1_bus_type      *pVM1_bus;
	MPI_AdapterHooks  *pHooks;

	virtual void   Clock();
	virtual void   Link_MPI( MPI_bus_type *pMPI, MPI_AdapterHooks *pHOOKS )
	{
		pVM1_bus = (VM1_bus_type*) pMPI;
		pHooks   = pHOOKS;
		qwClocksCount.Set_pArg( pVM1_bus->qwClocksCount );
	}
	
	bool   abReadAccess[0x10000];       //    .
	bool  abWriteAccess[0x10000];       //    .

	//  /  .
	virtual void IO_fun( IO_ModeType IO_Mode, IO_SizeType IO_Size, int nIO_Addr, word & wData ) {}	

	qword_ref  qwClocksCount;

	int   nRPLY_ON_Latency_CLK;
	int   nRPLY_OFF_Latency_CLK;
	
	int   nBusCycleDelay;
	word  wAddr;
	bool  WTBT_Prev;
	bool  DCLO_Prev;
	bool  INIT_Prev;

	word  IRQ_User_ID;		//     - IRQ_User_ID    ,  0.
	word  wVector;
	bool  bIRQ_ControllerActive;
	
	void  SetIRQ( word nVector ) { if( IRQ_User_ID ) { wVector = nVector; } }
	void  ClearIRQ()             { wVector = 0; }

	virtual void   Power_ON()  {}
	virtual void   Power_OFF() {}
	virtual void   Init() {}

	fastdelegate::FastDelegate0 <>   BusCycleController;

	void SYNC_ON_BusCycleController();
	void SYNC_OFF_BusCycleController();
	void SYNC_OFF_BusCycleController_1();

	void DIN_ON_BusCycleController();
	void DIN_OFF_BusCycleController();

	void DOUT_ON_BusCycleController();
	void DOUT_OFF_BusCycleController();

	void IRQ_BusCycleController_1();
	void IRQ_BusCycleController_2();
	void IRQ_BusCycleController_3();
	void IRQ_BusCycleController_4();
};

#endif