// rev 0.06

#pragma once
#include "defmacros.h"

#include "vm1a_pli.h"
#include "vm1a_plm.h"
#include "vm1g_pli.h"
#include "vm1g_plm.h"

#include "vm1_qbus_shared.h"

#include "Debugger.h"

#define DBG_OUT_PLM_LOG 1
#undef DBG_OUT_PLM_LOG

#define DBG_OUT_PLI_LOG 1
#undef DBG_OUT_PLI_LOG

#define DBG_OUT_WIRESDIFF 1
#undef DBG_OUT_WIRESDIFF

class C1801VM1 
{
	friend class CDebugger;
public:
	enum CPU_TYPE
	{
		CPU_VM1A,
		CPU_VM1G
	};

private:

	DWORD m_nDeviceID;

	// 
	//,    
	VM1_QBUS_REGS	reg, snap_reg;

	//wires,     ,     
	VM1_QBUS_WIRES	wire, prev_wire;

	//
	vm1a_pli		*pli_matrix_a;
	vm1a_plm		*plm_matrix_a;
	vm1g_pli		*pli_matrix_g;
	vm1g_plm		*plm_matrix_g;

	int				m_nClk;
	bool			m_bInit;
	MPI				*m_pMPI; //  ,        .

protected:
	CPU_TYPE		m_nCpuType;
	/*    
	: 
	  
	- 
	*/
	template <typename T> void data2bita(T d, bool *pBa, int nLen)
	{
		for (int i = 0; i < nLen; ++i)
		{
			pBa[i] = (d & 1) ? true:false;
			d >>= 1;
		}
	}

	// 
	template <typename T> void bita2data(bool *pBa, T &d, int nLen)
	{
		d = 0;
		for (int i = nLen - 1; i >= 0; i--)
		{
			d <<= 1;
			if (pBa[i])
			{
				d++;
			}
		}
	}

	void initial();

	void assign_func();
	void assign_all(const char *str = "  " );
	void eval_all_p();
	void eval_all_n();
	// ---   -------------------------------
	void always_posedge_pin_clk_p_or_posedge_pin_dclo_part_pin_clk_p();
	void always_posedge_pin_clk_p_or_posedge_pin_dclo_part_pin_dclo();
	void always_posedge_pin_clk_p_or_posedge_tve_reset_part_pin_clk_p();
	void always_posedge_pin_clk_p_or_posedge_tve_reset_part_tve_reset();
	void always_posedge_pin_clk_p_timer();
	void always_posedge_pin_clk_p_timer_part_pin_dclo();
	void always_posedge_pin_clk_p_timer_part_tve_reset();
	// ---------------------------------------------
	void always_posedge_pin_clk_n();
	void always_posedge_pin_clk_p();

	void always_posedge_pin_clk_n_or_posedge_mjres_part_mjres();
	void always_posedge_pin_clk_n_or_posedge_mjres_part_pin_clk_n();

	void always_posedge_pin_clk_p_or_posedge_mjres_part_mjres();
	void always_posedge_pin_clk_p_or_posedge_mjres_part_pin_clk_p();

	void always_posedge_pin_clk_n_or_negedge_qbus_tena_part_pin_clk_n();
	void always_posedge_pin_clk_n_or_negedge_qbus_tena_part_qbus_tena();

	void always_posedge_pin_clk_n_or_posedge_reset_part_reset();
	void always_posedge_pin_clk_n_or_posedge_reset_part_pin_clk_n();

	void always_posedge_pin_clk_p_or_posedge_reset_part_reset();
	void always_posedge_pin_clk_p_or_posedge_reset_part_pin_clk_p();

	void always_posedge_pin_clk_p_or_posedge_abort_part_abort();
	void always_posedge_pin_clk_p_or_posedge_abort_part_pin_clk_p();

	void always_a();

	// .
#ifdef DBG_OUT_PLI_LOG
	//	1.  PLI
	FILE *m_PLILog;
	void OutPLILog();
#endif
#ifdef DBG_OUT_PLM_LOG
	//	2.  PLM
	FILE *m_PLMLog;
	void OutPLMLog();
#endif


public:
	//input-output
	VM1_IO m_IOPins;	// .

	C1801VM1();
	~C1801VM1();

	void SetCpuType(CPU_TYPE nCPUType) 
	{
		m_nCpuType = nCPUType;
		
		if( nCPUType == CPU_VM1G )
		{
			reg.gpr[0] = 0077577;
			reg.gpr[1] = 0176777;
			reg.gpr[2] = 0001010;
			reg.gpr[3] = 0004042;
			reg.gpr[4] = 0011113;
			reg.gpr[5] = 0053756;
			reg.gpr[6] = 0041202;
		}
		else
		{
			reg.gpr[0] = 0133000;
			reg.gpr[1] = 0100514;
			reg.gpr[2] = 0100410;
			reg.gpr[3] = 0040000;
			reg.gpr[4] = 0100057;
			reg.gpr[5] = 0060000;
			reg.gpr[6] = 0000011;
		}
	}
	
	CPU_TYPE GetCpuType() { return m_nCpuType; }
	void initCPU(); //  .
	void AttachDebugger(CDebugger *pDbg) { pDbg->AttachDebugger(this); }
	void RegisterDevice(MPI *pMPI);

	void eval_p(int nClk);
	void eval_n(int nClk);

#ifdef DBG_OUT_WIRESDIFF
	void Out_Difference(int nClk, int n, const char *str = "  " );
	void compareBool(FILE *f, bool bOrig, bool bTest, const char *str);
	void compareByte(FILE *f, BYTE bOrig, BYTE bTest, const char *str);
	void compareWord(FILE *f, WORD bOrig, WORD bTest, const char *str);
	void compareDWord(FILE *f, DWORD bOrig, DWORD bTest, const char *str);
	void compareQWord(FILE *f, QWORD bOrig, QWORD bTest, const char *str);
	void compareBoolArray(FILE *f, bool *pOrig, bool *pTest, int nLen, const char *str);
#endif // DBG_OUT_WIRESDIFF

};

