// rev 0.06
#pragma once

struct VM1_QBUS_WIRES
{
	//input wire
	BYTE pin_pa;		// processor number
	bool pin_dclo;      // processor reset
	bool pin_aclo;      // power fail notification
	bool pin_irq[4];	// radial interrupt requests, pin_irq[0] == pin_virq// vectored interrupt request

	WORD pin_ad_in;		// data bus input
	bool pin_sync_in;   // address strobe input
	bool pin_dmgi;      // bus granted input
	bool	dma_in;			// 
	bool	plx[34];			//wire[33:0]	// main microcode matrix output
	bool	pli[11];			//wire[10:0]	// interrupt priority encode matrix output
	bool	tve_ack;			//wire			// VE-timer interrupt acknowledgement
	bool	exc_uop;			//wire			// Undefined opcode found
	bool	exc_oat;			//wire			// Odd address trap detector
	bool	exc_err7;			//wire			//
	bool	qbus_tena;			//wire			//
	bool	uop_ack;			//wire			// undefined instruction
	bool	aclo_ack;			//wire			// ACLO timer acknowlegement
	bool	irq2_ack;			//wire			// IRQ2 timer acknowlegement
	bool	irq3_ack;			//wire			// IRQ3 timer acknowlegement
	bool	ir_clr;				//wire			//
	bool	ir_seq_rc;			//wire			//
	bool	pli_req_rc;			//wire			// interrupt encoder request
	bool	plm_stb_rc;			//wire			// plm_register strobe source
	bool	plm1x_fc;			//wire			//
	bool	plm2x_fc;			//wire			// main matrix result strobes
	bool	plm_ena_fc;			//wire			//
	BYTE	plop;
	bool	uplr_stb;			//wire			//
	BYTE	plm23_fc;			//wire[3:0]		//
	BYTE	tplm_rc;			//wire[3:1]		//
	bool	mjres_rc;			//wire			//
	bool	reset_rc;			//wire			//
	bool	abort_rc;			//wire			//
	bool	sel_out;			//wire			//
	bool	dmr_out;			//wire			//
	bool	iako_out;			//wire			//
	bool	qbus_done;			//wire			//
	bool	rply_ack_fc;		//wire			//
	bool	oe_set_fc;			//wire			//
	bool	din_done;			//wire			//
	bool	dmr_req_rc;
	bool	dout_req_rc;
	bool	dout_start;
	bool	qbus_flag_rc;		//wire			//
	bool	sync_stb;			//wire			//
	bool	sync_fedge;			//wire			//
	bool	qbus_nosr_rc;		//wire			//
	bool	qbus_gnt;			//wire			//
	WORD	x, xr;				//wire[15:0]	// X bus
	WORD	y, yr;				//wire[15:0]	// Y bus
	bool	flag[4];			//wire[3:0]		//
	bool	cl_fc;				//wire			//
	bool	alu_b_fc;			//wire			//
	bool	alu_c_fc;			//wire			//
	bool	alu_d_fc;			//wire			//
	bool	alu_e_fc;			//wire			//
	bool	alu_x_fc;			//wire			//
	bool	alu_s_fc;			//wire			//
	bool	au_qsx, au_qsy;		//wire			// qbus temporary register
	bool	au_pswx, au_pswy;	//wire			// PSW read enable
	bool	au_pstbx;			//wire			// PSW write strobe
	bool	au_astb;			//wire			//
	bool	au_qstbx;			//wire			//
	bool	au_is0;				//wire			// ALU X argument strobes
//	bool	vc_vsel, vc_csel;	//wire
//	bool	rs0[14], rs1[14];	//wire[13:0]	// register select
//	bool	rsw[14];			//wire[13:0]	// register select
	WORD	rsw;								// register select

	// 
	bool	tve_reset;			//wire
	bool	tve_tclk128;		//wire			// prescaler /128 output
	bool	pin_spclk;			//wire			// external event on SP
	bool	tve_zero;			//wire			// timer counter zero value
	bool	tve_back;			//wire			// timer capture load


	void clear()
	{
		memset(this, 0, sizeof(VM1_QBUS_WIRES));
	};
	VM1_QBUS_WIRES &operator = (const VM1_QBUS_WIRES &src)
	{
		memcpy(this, &src, sizeof(VM1_QBUS_WIRES));
		return *this;
	};

	bool operator != (const VM1_QBUS_WIRES &src)
	{
		int n = memcmp(this, &src, sizeof(VM1_QBUS_WIRES));
		return (n != 0);
	};
	bool operator == (const VM1_QBUS_WIRES &src)
	{
		int n = memcmp(this, &src, sizeof(VM1_QBUS_WIRES));
		return (n == 0);
	};
};

struct VM1_QBUS_REGS
{
	QWORD	nPLR, nPLM;
	bool	plr[34];			//reg[33:0]		// main matrix result register (last stage)
	bool	plm[34];			//reg[33:0]		// main matrix result register (first stage)
	bool	mj[15];				//reg[14:0]		// microcode stage and condition register
	WORD	ir;					//reg[15:0]		// instruction register (last stage)
	WORD	ira;				//reg[15:0]		// instruction register (first stage)
	DWORD	rq;					//reg[19:0]		// interrupt request register
	bool	aclo;				//reg			// ACLO falling edge detectors
	bool	acok;				//reg			// ACLO raising edge detectors
	bool	irq2;				//reg			// IRQ2 falling edge detectors
	bool	irq3;				//reg			// IRQ3 falling edge detectors
	bool	uop;				//reg			// undefined operation latch
	bool	qbto;				//reg			// I/O bus timeout & OAT latch
	bool	ivto;				//reg			// IAKO bus timeout latch
	bool	start_irq;			//reg			// raised by 1777x2 register write
	bool	exc_dbl[4];			//reg[3:0]		// double error detector
	BYTE	qbus_timer;			//reg[5:0]		// qbus timeout counter
	bool	qbus_tovf;			//reg			// Qbus timer overflow latch
	BYTE	vsel;				//reg[3:0]		// vector selection register
	BYTE	plir;				//reg[3:1]		// pli[10:8:6:4] outputs latch
	bool	plir0;				//reg[0]		// pli[10:8:6:4] outputs latch
	bool	ir_stb2;			//reg			// instruction register strobe
	bool	ir_set;				//reg			//
	bool	ir_stop;			//reg			//
	bool	ir_seq;				//reg			//
	bool	pli_req;			//reg			// source and latch
	bool	plm_stb;			//reg			// plm_register strobe latch
	bool	plm1x, plm2x;		//reg			//
	bool	plm1x_hl;			//reg			//
	bool	plm_ena;			//reg			//
	bool	sop_out[4];			//reg[3:0]		//
	bool	mj_stb1, mj_stb2;	//reg			//
	bool	psw_stb, psw_stbc;	//reg			//
	bool	psw_mj;				//reg			//
	bool	plm23_wait;			//reg			//
	bool	plm23_ichk;			//reg			//
	BYTE	tplm;			//reg[3:1]		//
	BYTE	tplmz;			//reg[3:1]		//
	bool	plrt[9];			//reg[8:6]		//
	bool	plrtz[9];			//reg[8:6]		//
	bool	reset;				//reg			// global device hardware reset
	bool	abort;				//reg			// odd address. bus hang, invalid opcode
	bool	mjres;				//reg			// reset microcode state machine
	bool	mjres_h;			//reg			//
	bool	abort_tm;			//reg			//
	BYTE	sel_reg;			//reg			// peripheral block selectors
	bool	sel_xx;				//				//
	bool	rply_s2, rply_s3;	//reg			// disable rply generation if start_irq active
	bool	dmr_req;			//reg			//
	bool	dmr_req_l;			//reg			//
	bool	iako_out_lh;		//reg			//
	bool	iak_flag;			//reg			//
	bool	init_out[3];		//reg[2:0]		//
	bool	rply_ack[4];		//reg[3:0]		//
	bool	iako_oe;			//reg			//
	bool	oe_set;				//reg			//
	bool	oe_clr;				//reg			//
	bool	qbus_aseq[2];		//reg[1:0]		// address request sequencer
	bool	dout_done;			//reg			//
	bool	dout_req;			//reg			//
	bool	dout_out;			//reg			//
	bool	dout_out_l;			//reg			//
	bool	din_out;			//reg			//
	bool	din_out_l;			//reg			//
	bool	dmgi_in_l;			//reg			//
	bool	qbus_adr;			//reg			//
	bool	qbus_req;			//reg			//
	bool	qbus_win;			//reg			//
	bool	qbus_win_h;			//reg			//
	bool	qbus_own_fp;		//reg			//
	bool	qbus_own_rp;		//reg			//
	bool	qbus_flag;			//reg			//
	bool	sack_out;			//reg			//
	bool	sync_out;			//reg			//
	bool	sync_out_h;			//reg			//
	bool	sync_ena;			//reg			//
	bool	qbus_nosr_h;		//reg			//
	bool	qbus_gnt_l;			//reg			//
	bool	qbus_free_h;		//reg			//
	BYTE	reg_csr;			//reg[2:0]		// 177700: control register
	BYTE	reg_err;			//reg[7:0]		// 177704: error register
	bool	freg[4];			//reg[3:0]		//
	bool	psw[16];			//reg[15:0]		// processor status word
	bool	cl;					//reg			// inserted least bit carry
	bool	alu_b, alu_c;		//reg			// ALU function selector
	bool	alu_d, alu_e;		//reg			// registered for speed
	bool	alu_s, alu_x;		//reg			//
	bool	ustb, ustb_h;		//reg			//
	bool	ustb1, ustb1_h;		//reg			//
	bool	ustb1_hl;			//reg			//
	bool	alu_busy_fp;		//reg			// ALU busy (not ready phase shifted)
	bool	alu_busy_rp;		//reg			//
	bool	alu_nrdy;			//reg			// ALU not ready
	bool	au_alsl, au_alsh;	//reg			// ALU result strobes
	bool	au_ta0;				//reg			//
	bool	au_astb_xa;			//reg			// au_astb/au_qstb speed optimizers
	bool	au_qstb_xa;			//reg			//
	bool	au_astb_xu;			//reg			//
	bool	au_qstb_xu;			//reg			//
	WORD	qreg;				//reg[15:0]		// ALU Q register (Q-bus data)
	WORD	areg;				//reg[15:0]		// ALU A register (Q-bus address)
	WORD	xreg;				//reg[15:0]		// ALU X parameter register
	WORD	yreg;				//reg[15:0]		// ALU Y parameter register
	BYTE	vc_vreg;			//reg[3:0]
	WORD	gpr[14];			//reg[15:0]		// register array
	WORD	vmux;				//reg[15:0]		// variable for vector multiplexer
	WORD	cmux;				//reg[15:0]		// variable for constant multiplexer
	BYTE	xadr;				//reg[3:0]
	BYTE	yadr;				//reg[3:0]

	// 
	WORD	tve_limit;			//reg[15:0]		// timer reload & capture register
	WORD	tve_count;			//reg[15:0]		// timer counter register
	//bool	tve_csr[8];			//reg[7:0]		// timer control register
	BYTE	tve_csr;			//reg[7:0]		// timer control register
	BYTE	tve_pre;			//reg[6:0]		// clock prescaler counter
	BYTE	tve_div;			//reg[5:0]		// timer clock divisor
	BYTE	tve_edge;
	bool	tve_tclk;			//reg			// selected timer clock
	bool	tve_tclk4;			//reg			// prescaler /4 output
	bool	tve_intrq[2];		//reg[1:0]		// hidden interrupt request
	bool	tve_zprev;			//reg			// zero value edge detector

	void clear()
	{
		memset(this, 0, sizeof(VM1_QBUS_REGS));
	};
	VM1_QBUS_REGS &operator = (const VM1_QBUS_REGS &src)
	{
		memcpy(this, &src, sizeof(VM1_QBUS_REGS));
		return *this;
	};
	bool operator != (const VM1_QBUS_REGS &src)
	{
		int n = memcmp(this, &src, sizeof(VM1_QBUS_REGS));
		return (n != 0);
	};
	bool operator == (const VM1_QBUS_REGS &src)
	{
		int n = memcmp(this, &src, sizeof(VM1_QBUS_REGS));
		return (n == 0);
	};
};

//input-output,   ,    ,     
struct VM1_IO
{
	bool pin_ena;			//input			// processor clock enable
	BYTE pin_pa_n;			//input[1:0]	// processor number
	bool pin_sp_n;			//input			// peripheral timer input

	void clear()
	{
		pin_ena	   = true;	//  (false - )
		pin_pa_n   = ~(0);	//    (0..3)
		pin_sp_n   = true;	//  -
	};
};

