#include "stdafx.h"
#include "vm1a_pli.h"

#include "defmacros.h"

vm1a_pli::vm1a_pli(void)
	: pResult_Matrix(NULL)
	, m_nPrevRq(0xf0000000) //,       rq
{
	pResult_Matrix = new WORD[1 << 20];
	ASSERT(pResult_Matrix);

	if (pResult_Matrix)
	{
		for (int i = 0; i < (1 << 20); ++i)
		{
			pResult_Matrix[i] = calculateMatrix(i);
		}
	}

}

vm1a_pli::~vm1a_pli(void)
{
	if (pResult_Matrix)
	{
		delete pResult_Matrix;
	}
}

void vm1a_pli::eval(DWORD rq, bool *sp)
{
	if (m_nPrevRq == rq)
	{
		return;
	}
	m_nPrevRq = rq;

	WORD res = pResult_Matrix[rq];
	for (int i = 0; i < 11; ++i)
	{
		sp[i] = (res & g_WordBitMask[i]) ? true : false;
	}
}

WORD vm1a_pli::calculateMatrix(DWORD n)
{
	bool	p[21];

	bool psw10 = (bool)((n >> 0 ) & 1);
	bool plir  = (bool)((n >> 1 ) & 1);
	bool psw11 = (bool)((n >> 2 ) & 1);
	bool uerr  = (bool)((n >> 3 ) & 1);
	bool psw7  = (bool)((n >> 4 ) & 1);
	bool virq  = (bool)((n >> 8 ) & 1);       // low active level
	bool qbto  = (bool)((n >> 9 ) & 1);
	bool dble  = (bool)((n >> 10) & 1);
	bool aclo  = (bool)((n >> 11) & 1);
	bool wcpu  = (bool)((n >> 12) & 1);      // low active level
	bool acok  = (bool)((n >> 13) & 1);
	bool irq1  = (bool)((n >> 14) & 1);
	bool psw4  = (bool)((n >> 15) & 1);
	bool irq2  = (bool)((n >> 16) & 1);
	bool iato  = (bool)((n >> 17) & 1);
	bool irq3  = (bool)((n >> 18) & 1);

	p[0]    =  plir                              && !qbto && !aclo && !uerr &&  wcpu          && !irq1 && !irq2 && !irq3 &&  virq;
	p[10]   =  plir           && !psw10          && !qbto && !aclo && !uerr &&  wcpu          && !irq1;
	p[3]    =  plir && !psw11 && !psw10 &&  psw4 && !qbto && !aclo && !uerr &&  wcpu          &&  irq1;
	p[2]    =  plir && !psw11 && !psw10 && !psw4 && !qbto && !aclo && !uerr                   &&  irq1;
	p[1]    =  plir           && !psw10          && !qbto && !aclo && !uerr &&  wcpu && !psw7 && !irq1 &&  irq2;
	p[7]    =  plir           && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1 &&  irq2;
	p[6]    =  plir           && !psw10 &&  psw4 && !qbto && !aclo && !uerr &&  wcpu && !psw7 && !irq1 && !irq2 &&  irq3;
	p[9]    =  plir           && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1 && !irq2 &&  irq3;
	p[4]    =  plir           && !psw10 &&  psw4 && !qbto && !aclo && !uerr &&  wcpu && !psw7 && !irq1 && !irq2 && !irq3 && !virq;
	p[5]    =  plir           && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1 && !irq2 && !irq3 && !virq;
	p[11]   =  plir           && !psw10 && !psw4 && !qbto &&  aclo && !uerr;
	p[12]   =  plir           && !psw10 &&  psw4 && !qbto          && !uerr && !wcpu;
	p[13]   =  plir           && !psw10 &&  psw4 && !qbto &&  aclo && !uerr &&  wcpu;
	p[20]   =  plir && !psw11 && !psw10 && !iato &&  qbto && !dble;
	p[14]   =  plir &&  psw11           && !iato &&  qbto && !dble;
	p[18]   =  plir           &&  psw10 && !iato &&  qbto && !dble;
	p[19]   =  plir                     && !iato &&  qbto &&  dble;
	p[16]   =  plir                     &&  iato &&  qbto;
	p[15]   =  plir                     &&  uerr && !qbto;
	p[8]    = !plir && !acok;
	p[17]   = !plir &&  acok;

	WORD nRes = 0;

	nRes |= (!(p[20] || p[19] || p[15] || p[9] || p[7] || p[6] || p[1])) << 0;
	nRes |= (!(p[20] || p[19] || p[17] || p[13] || p[11] || p[9]  || p[6])) << 1;
	nRes |= (!(p[20] || p[17] || p[16] || p[5]  || p[4])) << 2;
	nRes |= (!(p[20] || p[19] || p[17] || p[16] || p[15] || p[12])) << 3;
	nRes |= (!(p[8])) << 4;
	nRes |= (!(p[20] || p[17] || p[15] || p[13] || p[12] || p[11] || p[9]  || p[7]  || p[6]  || p[5]  || p[4] || p[1])) << 5;
	nRes |= (!(         p[19] || p[18] || p[17] || p[16] || p[14] || p[3]  || p[2])) << 7;
	nRes |= (!(p[14] || p[13] || p[12] || p[11] || p[10] || p[9]  || p[8]  || p[7]  || p[5]  || p[3]  || p[2] || p[0])) << 9;
	bool bTmp2 =        p[20] || p[19] || p[18] ||          p[16] || p[15] || p[14];
	nRes |= (!(bTmp2 || p[12] || p[10] || p[9] || p[7] || p[5])) << 6;
	bool bTmp1 = bTmp2        || p[17] || p[13] || p[11];
	nRes |= (!(bTmp1 || p[9]  || p[6])) << 8;
	nRes |= (!(bTmp1 || p[12] || p[5] || p[4])) << 10;
	return nRes;
}

