#include "stdafx.h"
#include "vm1g_pli.h"

#include "defmacros.h"

vm1g_pli::vm1g_pli()
	: pResult_Matrix(NULL)
	, m_nPrevRq(0xf0000000) //,       rq
{
	pResult_Matrix = new WORD[1 << 20];
	ASSERT(pResult_Matrix);

	if (pResult_Matrix)
	{
		for (int i = 0; i < (1 << 20); ++i)
		{
			pResult_Matrix[i] = calculateMatrix(i);
	}
	}
}


vm1g_pli::~vm1g_pli()
{
	if (pResult_Matrix)
	{
		delete pResult_Matrix;
	}
}

void vm1g_pli::eval(DWORD rq, bool *sp)
{
	if (m_nPrevRq == rq)
	{
		return;
	}
	m_nPrevRq = rq;

	WORD res = pResult_Matrix[rq];
	for (int i = 0; i < 11; ++i)
	{
		sp[i] = (res & g_WordBitMask[i]) ? true : false;
	}
}

WORD vm1g_pli::calculateMatrix(DWORD n)
{
		bool	p[22];
		bool	pl[11];

	bool psw10 = (bool)((n >> 0 ) & 1);
	bool plir  = (bool)((n >> 1 ) & 1);
	bool psw11 = (bool)((n >> 2 ) & 1);
	bool uerr  = (bool)((n >> 3 ) & 1);
	bool psw7  = (bool)((n >> 4 ) & 1);
	bool virq  = (bool)((n >> 8 ) & 1);       // low active level
	bool qbto  = (bool)((n >> 9 ) & 1);	// qbus timeouts and odd address
	bool dble  = (bool)((n >> 10) & 1);
	bool aclo  = (bool)((n >> 11) & 1);	// ACLO detector requests
	bool wcpu  = (bool)((n >> 12) & 1);      // low active level
	bool acok  = (bool)((n >> 13) & 1);	// ACLO detector requests
	bool irq1  = (bool)((n >> 14) & 1);
	bool psw4  = (bool)((n >> 15) & 1);
	bool irq2  = (bool)((n >> 16) & 1);
	bool iato  = (bool)((n >> 17) & 1);	// qbus timeouts and odd address
	bool irq3  = (bool)((n >> 18) & 1);
	bool tve   = (bool)((n >> 19) & 1);

//______________________________________________________________________________
//
// wcpu = 0, simplified matrix to demostrate priority encoding
//
//                             &  iato   &  qbto;
//                             & ~iato   &  qbto &  dble;


//          psw11            & ~iato   &  qbto & ~dble;
//                  &  psw10 & ~iato   &  qbto & ~dble;
//         ~psw11   & ~psw10 & ~iato   &  qbto & ~dble;
//
//                                uerr   & ~qbto;

//                    ~psw10 &  psw4 & ~qbto         & ~uerr;
//                    ~psw10 & ~psw4 & ~qbto &  aclo & ~uerr;
//         ~psw11   & ~psw10 & ~psw4 & ~qbto & ~aclo & ~uerr         &  irq1;
//   tve &          & ~psw10 & ~psw4 & ~qbto & ~aclo & ~uerr & ~psw7 & ~irq1;
//  ~tve &          & ~psw10 & ~psw4 & ~qbto & ~aclo & ~uerr & ~psw7 & ~irq1 &  irq2;
//  ~tve &          & ~psw10 & ~psw4 & ~qbto & ~aclo & ~uerr & ~psw7 & ~irq1 & ~irq2 &  irq3;
//  ~tve &          & ~psw10 & ~psw4 & ~qbto & ~aclo & ~uerr & ~psw7 & ~irq1 & ~irq2 & ~irq3 & ~virq;
//
	p[0]  = !tve && plir                                    && !qbto && !aclo && !uerr &&  wcpu            && !irq1 && !irq2 && !irq3 && virq;
	p[10] =         plir             && !psw10 && !qbto                     && !uerr &&  wcpu            && !irq1;
	p[4]  =         plir             && !psw10 && psw4  && !qbto          && !uerr &&  wcpu            &&  irq1;

	p[1]  =         plir && !psw11 && !psw10 && !psw4 && !qbto && !aclo && !uerr                     &&  irq1;
	p[6]  =  tve && plir             && !psw10 &&  psw4 && !qbto          && !uerr &&  wcpu && !psw7 && !irq1;
	p[8]  =  tve && plir             && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1;
	p[13] = !tve && plir             && !psw10 &&  psw4 && !qbto          && !uerr &&  wcpu && !psw7 && !irq1 &&  irq2;
	p[3]  = !tve && plir             && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1 &&  irq2;
	p[14] = !tve && plir             && !psw10 &&  psw4 && !qbto          && !uerr &&  wcpu && !psw7 && !irq1 && !irq2 &&  irq3;
	p[9]  = !tve && plir             && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1 && !irq2 &&  irq3;
	p[2]  = !tve && plir             && !psw10 &&  psw4 && !qbto          && !uerr &&  wcpu && !psw7 && !irq1 && !irq2 && !irq3 && !virq;
	p[5]  = !tve && plir             && !psw10 && !psw4 && !qbto && !aclo && !uerr          && !psw7 && !irq1 && !irq2 && !irq3 && !virq;

	p[11] =         plir             && !psw10 &&  psw4 && !qbto          && !uerr && !wcpu;
	p[12] =         plir             && !psw10 && !psw4 && !qbto &&  aclo && !uerr;
	p[21] =         plir && !psw11 && !psw10 && !iato   &&  qbto && !dble;
	p[15] =         plir &&  psw11 && !iato    &&  qbto   && !dble;
	p[19] =         plir             &&  psw10 && !iato   &&  qbto && !dble;
	p[20] =         plir                         && !iato   &&  qbto &&  dble;
	p[17] =         plir                         &&  iato   &&  qbto;
	p[16] =         plir                         &&  uerr   && !qbto;
	p[7]  =        !plir && !acok;
	p[18] =        !plir &&  acok;

	pl[0]  = p[21] || p[20] || p[16] || p[14] || p[13] || p[9]  || p[8]  || p[6]  || p[3]  || p[2];
	pl[1]  = p[21] || p[20] || p[18] || p[14] || p[12] || p[9]  || p[8]  || p[6];
	pl[2]  = p[21] || p[18] || p[17] || p[5]  || p[2];
	pl[3]  = p[21] || p[20] || p[18] || p[17] || p[16] || p[11];
	pl[4]  = p[7];
	pl[5]  = p[21] || p[18] || p[16] || p[14] || p[13] || p[12] || p[11] || p[9]  || p[8]  || p[6] || p[5] || p[3] || p[2];
	pl[6]  = p[21] || p[20] || p[19] || p[17] || p[16] || p[15] || p[14] || p[13] || p[11] || p[9] || p[5] || p[3] || p[2];
	pl[7]  = p[20] || p[19] || p[18] || p[17] || p[15] || p[4]  || p[1];
	pl[8]  = p[21] || p[20] || p[19] || p[18] || p[17] || p[16] || p[15] || p[14] || p[12] || p[9] || p[8] || p[6];
	pl[9]  = p[14] || p[13] || p[12] || p[11] || p[10] || p[9]  || p[8]  || p[7]  || p[5]  || p[4] || p[3] || p[1] || p[0];
	pl[10] = p[21] || p[20] || p[19] || p[18] || p[17] || p[16] || p[15] || p[12] || p[11] || p[5] || p[2];
	
	WORD d = 0;
	for (int i = 10; i >= 0; i--)
	{
		d <<= 1;
		if (!pl[i])
		{
			d++;
		}
	}
	return d;
}

